## LIBRARIES TO USE
library(foreign)

###########  HODRICK PRESCOTT FILTER  #########################
## QUARTERLY: lambda=1600
## ANNUAL: lambda=6.25
hpfilter <- function(x,lambda=6.25){
eye <- diag(length(x))
result <- solve(eye+lambda*crossprod(diff(eye,lag=1,d=2)),x)
return(result)
}

## READ DATA
data <- read.csv("ltfrp_us.csv", header=T)

## DEFINE VARIABLES
logr <- log(data$R)
year <- data$AN
roe <- data$X - hpfilter(data$X)
dw <- data$W_R - hpfilter(data$W_R)
urate <- data$URATE
kdef <- data$PKK
uratel1 <- data$URATEL1
kdefl1 <- data$PKKL1

## DEMEAN STATIONARY VARIABLES
roe <- roe - mean(roe)
dw <- dw - mean(dw)


################ REGRESSION  ##################

reg11 <- lm(logr~year+roe+dw+urate+kdef)   

########## ESTIMATION OF SECOND MOMENTS ###############
#### SIGMA1
sigma1 <- sqrt((sum(roe^2))/(length(roe)))

#### SIGMA2
sigma2 <- sqrt((sum(dw^2))/(length(dw)))

#### SIGMA3
ur.diff <- (urate[-1]-uratel1[-1])
sigma3 <- sqrt((sum(ur.diff^2))/(length(ur.diff)))

#### SIGMA4
kdef.diff <- (kdef[-1]-kdefl1[-1])
sigma4 <- sqrt((sum(kdef.diff^2))/(length(kdef.diff)))

#### SIGMA12
sigma12 <- (sum(roe*dw))/(length(roe))




########## ESTIMATION OF LAMBDA3 (NEWEY-WEST) ###############

uu <- ur.diff
T <- length(uu)  

gamma0 <- 0
for (i in 1:T) {
  gamma0 <- gamma0 + uu[i]*uu[i]
}
gamma0 <- gamma0/T

gamma1 <- 0
for (i in 2:T) {
  gamma1 <- gamma1 + uu[i]*uu[i-1]
}
gamma1 <- gamma1/T

gamma2 <- 0
for (i in 3:T) {
  gamma2 <- gamma2 + uu[i]*uu[i-2]
}
gamma2 <- gamma2/T


gamma3 <- 0
for (i in 4:T) {
  gamma3 <- gamma3 + uu[i]*uu[i-3]
}
gamma3 <- gamma3/T

lambda3 <- sqrt(gamma0 + 2*(3/4)*gamma1 + 2*(1/2)*gamma2 + 2*(1/4)*gamma3)


########## ESTIMATION OF LAMBDA4 (NEWEY-WEST) ###############

uu <- kdef.diff
T <- length(uu)  

gamma0 <- 0
for (i in 1:T) {
  gamma0 <- gamma0 + uu[i]*uu[i]
}
gamma0 <- gamma0/T

gamma1 <- 0
for (i in 2:T) {
  gamma1 <- gamma1 + uu[i]*uu[i-1]
}
gamma1 <- gamma1/T

gamma2 <- 0
for (i in 3:T) {
  gamma2 <- gamma2 + uu[i]*uu[i-2]
}
gamma2 <- gamma2/T


gamma3 <- 0
for (i in 4:T) {
  gamma3 <- gamma3 + uu[i]*uu[i-3]
}
gamma3 <- gamma3/T

lambda4 <- sqrt(gamma0 + 2*(3/4)*gamma1 + 2*(1/2)*gamma2 + 2*(1/4)*gamma3)


########## ESTIMATION OF LAMBDA5 (NEWEY-WEST) ###############

uu <- (ur.diff + kdef.diff)
T <- length(uu)  

gamma0 <- 0
for (i in 1:T) {
  gamma0 <- gamma0 + uu[i]*uu[i]
}
gamma0 <- gamma0/T

gamma1 <- 0
for (i in 2:T) {
  gamma1 <- gamma1 + uu[i]*uu[i-1]
}
gamma1 <- gamma1/T

gamma2 <- 0
for (i in 3:T) {
  gamma2 <- gamma2 + uu[i]*uu[i-2]
}
gamma2 <- gamma2/T


gamma3 <- 0
for (i in 4:T) {
  gamma3 <- gamma3 + uu[i]*uu[i-3]
}
gamma3 <- gamma3/T

lambda5 <- sqrt(gamma0 + 2*(3/4)*gamma1 + 2*(1/2)*gamma2 + 2*(1/4)*gamma3)


########## ESTIMATION OF LAMBDA (NEWEY-WEST) ###############

uu <- reg11$residuals  ## RESIDUALS FROM REGRESSION
T <- length(reg11$residuals)  

gamma0 <- 0
for (i in 1:T) {
  gamma0 <- gamma0 + uu[i]*uu[i]
}
gamma0 <- gamma0/T

gamma1 <- 0
for (i in 2:T) {
  gamma1 <- gamma1 + uu[i]*uu[i-1]
}
gamma1 <- gamma1/T

gamma2 <- 0
for (i in 3:T) {
  gamma2 <- gamma2 + uu[i]*uu[i-2]
}
gamma2 <- gamma2/T


gamma3 <- 0
for (i in 4:T) {
  gamma3 <- gamma3 + uu[i]*uu[i-3]
}
gamma3 <- gamma3/T

lambda <- sqrt(gamma0 + 2*(3/4)*gamma1 + 2*(1/2)*gamma2 + 2*(1/4)*gamma3)


########### DISTRIBUTION OF TEST STATISTIC ###########################

## NUMBER OF ITERATIONS
N <- 100000

## INITIALIZE STATISTIC
nonstd.stat <- rep(0,N)

## MAIN LOOP
for (i in 1:N) {

## SAMPLE SIZE	
T <- 2007-1948+1
	## GENERATE RANDOM WALK
	zeta <- rep(0, times=T)
	u <- rnorm(T, mean=0, sd=1)
	for (t in 1:(T-1)) {
  	zeta[t+1] <- zeta[t] + u[t+1]
	}	
	
zeta_1 <- zeta[-length(zeta)]
u_1 <- u[-1]
count <- 1:(T-1)
intWr <- (sum(zeta_1))/((T-1)^(3/2))
intrWr <- (sum(count*zeta_1))/((T-1)^(5/2))
intWrsq <- (sum(zeta_1^2))/((T-1)^(2))
W1 <- (sum(u_1))/((T-1)^(1/2))

#### Q-MATRIX
q11 <- 1
q12 <- 1/2
q13 <- 0
q14 <- 0
q15 <- lambda3 * intWr
q16 <- lambda4 * intWr

q21 <- q12
q22 <- 1/3
q23 <- 0
q24 <- 0
q25 <- lambda3 * intrWr
q26 <- lambda4 * intrWr

q31 <- q13
q32 <- q23
q33 <- sigma1^2
q34 <- sigma12
q35 <- 0
q36 <- 0

q41 <- q14
q42 <- q24
q43 <- q34
q44 <- sigma2^2
q45 <- 0
q46 <- 0

q51 <- q15
q52 <- q25
q53 <- q35
q54 <- q45
q55 <- lambda3^2*intWrsq
q56 <- (1/2)*(lambda5^2 - lambda3^2 - lambda4^2)*intWrsq

q61 <- q16
q62 <- q26
q63 <- q36
q64 <- q46
q65 <- q56
q66 <- (lambda4^2)*(intWrsq)

## DETERMINANT OF Q-MATRIX
detQ <- q11*(q22*(q33*q44*(q55*q66-q56)-q34*(q55*q66-q56))-q25*(q33*q44*(q25*q66-q26*q56)-q34*q34*(q25*q66-q26*q56))+q26*(q33*q44*(q25*q56-q26*q55)-q34*q34*(q25*q56-q26*q55)))-q12*(q12*(q33*q44*(q55*q66-q56*q56)-q34*q34*(q55*q66-q56*q56))-q25*(q33*q44*(q15*q66-q16*q56)-q34*q34*(q15*q66-q16*q56))+q26*(q33*q44*(q15*q56-q16*q55)-q34*q34*(q15*q56-q16*q55)))+q15*(q12*(q33*q44*(q25*q66-q26*q56)-q34*q34*(q25*q66-q26*q56))-q22*(q33*q44*(q15*q66-q16*q56)-q34*q34*(q15*q66-q16*q56))+q26*((q15*q26-q16*q25)*q33*q44-(q15*q26-q16*q25)*q34*q34))-q16*(q12*(q33*q44*(q25*q56-q26*q55)-q34*q34*(q25*q56-q26*q55))-q22*(q33*q44*(q15*q56-q16*q55)-q34*q34*(q15*q56-q16*q55))+q25*((q15*q26-q16*q25)*q33*q44-(q15*q26-q16*q25)*q34*q34))

#### A-VECTOR
a1 <- lambda * W1
a2 <- lambda * ( W1 - intWr)
a3 <- sigma1*sqrt(gamma0)*W1
a4 <- sigma2*sqrt(gamma0)*W1
a5 <- (1/2)*( (lambda^2 - sigma3^2)*intWrsq - lambda^2*W1^2 )
a6 <- (1/2)*( (lambda^2 - sigma4^2)*intWrsq - lambda^2*W1^2 )

#### SECOND ROW OF ADJOINT OF Q-MATRIX
p21 <- -q12*(q33*q44*(q55*q66-q56)-q34*(q55*q66-q56))+q25*(q33*q44*(q15*q66-q16*q56)-q34*q34*(q15*q66-q16*q56))-q26*(q33*q44*(q15*q56-q16*q55)-q34*q34*(q15*q56-q16*q55))

p22 <- q11*(q33*q44*(q55*q66-q56*q56)-q34*q34*(q55*q66-q56*q56))-q15*(q33*q44*(q15*q66-q16*q56)-q34*q34*(q15*q66-q16*q56))+q16*(q33*q44*(q15*q56-q16*q55)-q34*q34*(q15*q56-q16*q55))

p23 <- 0

p24 <- 0

p25 <- -q11*(q25*(q33*q44*q66-q34*q34*q66)-q26*(q33*q44*q56-q34*q34*q56))+q15*(q12*(q33*q44*q66-q34*q34*q66)-q26*(q16*q33*q44-q16*q34*q34))-q16*(q12*(q33*q44*q56-q34*q34*q56)-q25*(q16*q33*q44-q16*q34*q34))

p26 <- q11*(q25*(q33*q44*q56-q34*q34*q56)-q26*(q33*q44*q55-q34*q34*q55))-q15*(q12*(q33*q44*q56-q34*q34*q56)-q26*(q15*q33*q44-q15*q34*q34))+q16*(q12*(q33*q44*q55-q34*q34*q55)-q25*(q15*q33*q44-q15*q34*q34))


## STATISTIC
nonstd.stat[i] <- (p21*a1 + p22*a2 + p23*a3 + p24*a4 + p25*a5 + p26*a6)/detQ 
cat("Iteration:", i, ";Value of statistic:", nonstd.stat[i], "\n")
}

################## RESULTS ########################
cat("-------------------------------------", "\n")
cat("--------- REGRESSION RESULTS --------", "\n")
print(summary(reg11))

test.statistic <- (T)^(3/2)*(reg11$coefficient[2])
cat("--------- TEST STATISTIC ------------", "\n")
cat("Test Statistic =", test.statistic, "\n")
cat("--------- CRITICAL VALUES -----------", "\n")

print( round(quantile(nonstd.stat,  probs=c(1,2.5,5,10,50,90,95,97.5,99)/100), 4))

 
